--[[

This file is part of RampageAI.

RampageAI is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

RampageAI is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with RampageAI; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

--]]

----------------------------
-- Vanilmirth-specific AI --
----------------------------

--[[
	Unsupported Skills:

	Bio-explosion ( ID 8016, SP ? )
--]]


--== Vanilmirth variables ==--

-- Chaotic Blessings table
VanilCBTable = {
	--	Self,	Owner,	Monster
	{	20,	30,	50 },
	{	50,	10,	40 },
	{	25,	50,	25 },
	{	60,	04,	36 },
	{	34,	34,	34 }		-- rounded up all chances
}

-- Most efficient level for healing owner
VanilCBOwner = 0

-- Most efficient level for healing homunculus
VanilCBHomun = 0

-- Last amount of SP before Chaotic Blessings was casted
VanilCB_PrevSP = 0


-- Initialization function
function Vanilmirth_Init()
	-- Add the Caprice skill to the attack skill list
	AddAttackSkill(8013,{ 22, 24, 26, 28, 30 },10,function(level) return 800 + (level * 200) end)

	-- Add the heal-owner function
	if HealSkills and (ChaoticBlessingsOwnerHP > 0 or
		ChaoticBlessingsHomunHP > 0) then

		-- Set the function to check if we need to use it
		HomunSkillCheck = Vanilmirth_HealSkillCheck

		-- Get the max level Chaotic Blessings
		local max = MaxChaoticBlessings

		-- Check to see if we're allowed to heal the alchemist with Chaotic Blessings
		if ChaoticBlessingsOwnerHP > 0 then
			-- Yep!
			CB_CheckOwner = Vanilmirth_CheckOwner

			-- Decide the most efficient level of Chaotic Blessings to use
			-- in order to heal the alchemist
			VanilCBOwner = Vanilmirth_FindMostEfficient(2)

			-- Log the most efficient owner level
			Log(0,"Will be using Chaotic Blessings level %d to heal the alchemist",
				VanilCBOwner)
		else
			-- Nope!
			CB_CheckOwner = ZeroReturn
		end

		-- Check to see if we're allowed to heal the homunculus with Chaotic Blessings
		if ChaoticBlessingsHomunHP > 0 then
			-- We're allowed to
			CB_CheckHomun = Vanilmirth_CheckHomun

			-- Decide the most efficient level of Chaotic Blessings to use
			-- in order to heal the homunculus
			VanilCBHomun = Vanilmirth_FindMostEfficient(1)

			-- Log the most efficient homun level
			Log(0,"Will be using Chaotic Blessings level %d to heal the homunculus",
				VanilCBHomun)
		else
			-- We aren't allowed to
			CB_CheckHomun = ZeroReturn
		end
	end
end

-- Function to decide the most efficient skill level to use
function Vanilmirth_FindMostEfficient(id)
	-- Get the max level
	local max = MaxChaoticBlessings

	-- Loop through our table of Chaotic Blessings' chances
	local level,chance,mult
	local cur_level,cur_chance = 0,0
	for level=max,1,-1 do
		-- Get the chance for that level
		chance = VanilCBTable[level][id]

		-- Get the level multiplier
		mult = math.ceil(level / 2)

		-- Multiply the chance by the multiplier
		chance = chance * mult

		-- Check if this chance is above our current level
		if chance >= cur_chance then
			-- We found a more efficient level
			cur_level = level
			cur_chance = chance
		end
	end

	-- Return the one we found
	return cur_level
end

-- Chaotic Blessings function
function Vanilmirth_HealSkillCheck()
	-- Check to see if healing skills are allowed
	if not SkillAllowed(SKILL_TYPE_HEAL) then
		return false
	end

	-- Check if we have a target lock, and we're not allowed to heal while locked
	if Target.Lock and not ChaoticBlessingsWhileLocked then
		return false
	end

	-- Get the level to heal our owner with
	local level = CB_CheckOwner()
	local healowner = false

	-- Check to see if we don't need to heal our owner
	if level == 0 then
		-- Check if we should try to heal the homunculus
		level = CB_CheckHomun()
	else
		healowner = true
	end

	-- Check if level isn't 0
	if level ~= 0 then
		-- Check if we have enough SP
		local sp = SPUsable(HomunID)

		if sp >= 40 and sp ~= VanilCB_PrevSP then
			-- Get the thing we're healing
			local named = "homunculus"
			if healowner then
				named = "alchemist"
			end

			-- Log it
			Log(0,"Attempting to use Chaotic Blessings (level %d) to try to heal the %s.",
				level,named)

			-- Check if the homunculus is moving
			if HomunMotion == MOTION_MOVE then
				-- The skill usage would stop it
				BypassMove(HomunID,GetV(V_POSITION,HomunID))
			end

			-- Use Chaotic Blessings
			SkillObject(HomunID, level, 8014, HomunID)
		end

		-- Set the previous amount of SP
		-- 	(this is to make sure it doesn't spam more than it needs to)
		VanilCB_PrevSP = sp

		--[[ NOTE: If we don't have enough SP, we still want to
				pause the usage of the attack skills so that
				we can use the heal skill when we get enough SP
		--]]

		-- We are trying to use a healing skill, so pause attack skills
		return true
	end

	-- Not trying to use a skill
	return false
end

function Vanilmirth_CheckOwner()
	-- Check to see if our owner is below the HP value
	if ActorHP(OwnerID) <= ChaoticBlessingsOwnerHP then
		-- We should try to heal them
		return VanilCBOwner
	end

	-- Don't try to heal yet
	return 0
end

function Vanilmirth_CheckHomun()
	-- Check to see if our homunculus is below the HP value
	if ActorHP(HomunID) <= ChaoticBlessingsHomunHP then
		-- Try to heal the homunculus
		return VanilCBHomun
	end

	-- Don't try to heal yet
	return 0
end